<?php

namespace Drupal\email_content_templates\Plugin\EctMail;

use Drupal\Component\Plugin\PluginBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\email_content_templates\Entity\EmailContentTemplateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Base class for email_content_templates_info plugins.
 */
abstract class EctMailPluginBase extends PluginBase implements EctMailPluginInterface {

  use StringTranslationTrait;

  /**
   * The condition plugin manager.
   *
   * @var \Drupal\Core\Condition\ConditionManager
   */
  protected $conditionsManager;

  /**
   * The language manager service.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $language;

  /**
   * The plugin context handler.
   *
   * @var \Drupal\Core\Plugin\Context\ContextHandlerInterface
   */
  protected $contextHandler;

  /**
   * The context manager service.
   *
   * @var \Drupal\Core\Plugin\Context\ContextRepositoryInterface
   */
  protected $contextRepository;

  /**
   * The condition settings for this plugin.
   *
   * @var array
   */
  protected $conditions = [];

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
    );

    $instance->conditionsManager = $container->get('plugin.manager.condition');
    $instance->language = $container->get('language_manager');
    $instance->contextHandler = $container->get('context.handler');
    $instance->contextRepository = $container->get('context.repository');

    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function label() {
    // Cast the label to a string since it is a TranslatableMarkup object.
    return (string) $this->pluginDefinition['label'];
  }

  /**
   * {@inheritdoc}
   */
  public function paramTokenMap() {
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function applies(EmailContentTemplateInterface $template, array $message): bool {
    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function preRenderAlterMail(EmailContentTemplateInterface $template, array &$message): void {
  }

  /**
   * {@inheritdoc}
   */
  public function postRenderAlterMail(EmailContentTemplateInterface $template, array &$message): void {
  }

  /**
   * {@inheritdoc}
   */
  public function buildConditionsForm(EmailContentTemplateInterface $template, FormStateInterface $form_state) {
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function getPluginCollections() {
    return [
      'settings' => $this->getPluginCollection(),
      'conditions' => $this->getVisibilityConditions(),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getTestForm(): array {
    $param_token_map = $this->paramTokenMap();
    $form = [];
    foreach ($param_token_map as $param_name => $entity_type) {
      $form[$param_name] = [
        '#title' => $param_name,
        '#type' => 'entity_autocomplete',
        '#target_type' => $entity_type,
        '#required' => TRUE,
      ];
    }
    return $form;
  }

}
