<?php

namespace Drupal\email_content_templates\Plugin\EctMail;

use Drupal\Component\Plugin\PluginInspectionInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Plugin\ObjectWithPluginCollectionInterface;
use Drupal\email_content_templates\Entity\EmailContentTemplateInterface;

/**
 * Interface for email_content_templates_info plugins.
 */
interface EctMailPluginInterface extends PluginInspectionInterface, ObjectWithPluginCollectionInterface, ContainerFactoryPluginInterface {

  /**
   * Returns the translated plugin label.
   *
   * @return string
   *   The translated title.
   */
  public function label();

  /**
   * Returns the information so hook_mail parameters can be mapped to tokens.
   *
   * @return array
   *   The map.
   */
  public function paramTokenMap();

  /**
   * Returns the form used for testing the template.
   *
   * @return array
   *   The form array.
   */
  public function getTestForm(): array;

  /**
   * Checks if the given template should be used.
   *
   * @param \Drupal\email_content_templates\Entity\EmailContentTemplateInterface $template
   *   The template.
   * @param array $message
   *   The message.
   *
   * @return bool
   *   Whether the template should be used or not.
   */
  public function applies(EmailContentTemplateInterface $template, array $message): bool;

  /**
   * Alter the mail message before rendering the body.
   *
   * Can be used to add variables for rendering.
   *
   * @param \Drupal\email_content_templates\Entity\EmailContentTemplateInterface $template
   *   The template.
   * @param array $message
   *   The data from hook_mail().
   */
  public function preRenderAlterMail(EmailContentTemplateInterface $template, array &$message): void;

  /**
   * Alter the mail message after rendering the body.
   *
   * @param \Drupal\email_content_templates\Entity\EmailContentTemplateInterface $template
   *   The template.
   * @param array $message
   *   The data from hook_mail().
   */
  public function postRenderAlterMail(EmailContentTemplateInterface $template, array &$message): void;

  /**
   * Builds the conditions form.
   *
   * @param \Drupal\email_content_templates\Entity\EmailContentTemplateInterface $template
   *   The template using the conditions.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   *
   * @return array
   *   The form build.
   */
  public function buildConditionsForm(EmailContentTemplateInterface $template, FormStateInterface $form_state);

}
